import os
import gtk
import gobject
import sys
import re

from xml.sax import handler, make_parser

from filterpane import FilterPane
from detailspane import DetailsPane
from actionpane import ActionPane

size_folder_re=re.compile('[0-9]+x[0-9]+|scalable')

class LinkLoaderHandler(handler.ContentHandler):
    def __init__(self):
        self.inIcon = False
	self.inLink = False
        self.mapping = {}

    def startElement(self, name, attributes):
        if name == "context":
            self.Context = attributes["dir"]

        elif name == "icon":
            self.inIcon = True
            self.IconName = attributes["name"]

	elif name == "link":
	    self.inLink = True

    def characters(self, content):
        if self.inLink:
	    source_context = self.Context
	    source_name = self.IconName

	    s = os.path.split(content)
	    filename = s[1]
            split = filename.split('.')
	    ext = split.pop(-1)
	    target_name = '.'.join(split)
           
	    s = os.path.split(s[0])
	    if not s[1]:
	        target_context = source_context + '/'
            else:
	        target_context = ''
	        while s[1] != '..':
	            target_context = os.path.join(s[1], target_context)
                    s = os.path.split(s[0])

	    #remove the trailing slash
	    target_context = target_context[0:-1]

            key = (target_name, target_context)
            value = (source_name, source_context, ext) 
            print key,
            print '=',
            print value
	    self.mapping[key] = value 


    def endElement(self, name):
        if name == "icon":
            self.inIcon = False
	
	elif name == "link":
	    self.inLink = False

class CurrentThemeProcessor(gobject.GObject):
    COL_THEME=0
    COL_CONTEXT=1
    COL_NAME=2
    COL_DISPLAY_STR=3
    COL_DISPLAY_PIXBUF=4
    COL_BG_COLOR=5
    COL_IS_LINK=6
    COL_IS_NEW_LINK=7
    COL_EXTENTION=8
    COL_SIZE_START=9
    COL_SIZE_12=COL_SIZE_START
    COL_SIZE_16=COL_SIZE_START+1
    COL_SIZE_22=COL_SIZE_START+2
    COL_SIZE_24=COL_SIZE_START+3
    COL_SIZE_32=COL_SIZE_START+4
    COL_SIZE_36=COL_SIZE_START+5
    COL_SIZE_48=COL_SIZE_START+6
    COL_SIZE_64=COL_SIZE_START+7
    COL_SIZE_96=COL_SIZE_START+8
    COL_SIZE_128=COL_SIZE_START+9

    
    icon_themes = {} 
    icon_contexts = {}
    icon_new_links = {}

    icon_model = gtk.ListStore(gobject.TYPE_STRING, #theme
                               gobject.TYPE_STRING, #context
			       gobject.TYPE_STRING, #name
			       gobject.TYPE_STRING, #display string
			       gtk.gdk.Pixbuf,      #display pixbuf
			       gtk.gdk.Color,       #bg color
			       gobject.TYPE_BOOLEAN, #is this file a link
			       gobject.TYPE_BOOLEAN, #is this a new link made by the tool?
			       gobject.TYPE_STRING, #file extention
			       gtk.gdk.Pixbuf,      #size 12
			       gtk.gdk.Pixbuf,      #size 16
			       gtk.gdk.Pixbuf,      #size 22
			       gtk.gdk.Pixbuf,      #size 24
			       gtk.gdk.Pixbuf,      #size 32
			       gtk.gdk.Pixbuf,      #size 36
			       gtk.gdk.Pixbuf,      #size 48
			       gtk.gdk.Pixbuf,      #size 64
			       gtk.gdk.Pixbuf,      #size 96
			       gtk.gdk.Pixbuf       #size 128
			       )

    col_size_map = {12: COL_SIZE_12, 
                    16: COL_SIZE_16, 
		    22: COL_SIZE_22, 
		    24: COL_SIZE_24, 
		    32: COL_SIZE_32, 
		    36: COL_SIZE_36, 
		    48: COL_SIZE_48, 
		    64: COL_SIZE_64, 
		    96: COL_SIZE_96, 
		    128: COL_SIZE_128}

    context_list = []

    __gsignals__ = {
       'context-added' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                          (gobject.TYPE_STRING,))
       }

    def get_themes_list(self):
        list = self.icon_themes.keys()

	return list

    def get_context_list(self):
        list = self.icon_contexts.keys()

        return list

    def process_icon_queue(self):
        try:
            icon_name = self.unprocessed_icon_queue.pop(0)
	except:
	    return False

	bg_color = None
	icon = None
	display_name_str = icon_name + " ("

	pixbuf_list = []
	for i in range(len(self.col_size_map)):
	    pixbuf_list.append(None)
       
	icon_sizes = self.icon_theme.get_icon_sizes(icon_name)
	big_size = 0
	display_pixbuf = None
	is_link = False
	for size in icon_sizes:
	    if size == -1:
	       continue

	    display_name_str = display_name_str + "%i "%(size)

	    if not self.col_size_map.has_key(size):
		sys.stderr.write ("Warning *** Icon (%s) size %ix%i is not a standard size\n"%(icon_name, size, size))
		continue

	    icon_info = self.icon_theme.lookup_icon(icon_name, size, 0)
	    if not icon:
		s = os.path.split(icon_info.get_filename()) #file
		file = s[1]
		file_split = file.split('.')
		if len(file_split) > 1:
		    ext = file_split[-1]
		else:
		    ext = ''

		s = os.path.split(s[0]) #context
		context = '' 

                #prepend the context until we hit the size directory
                m = size_folder_re.match(s[1])
                while not m:
		    context = os.path.join(s[1], context)
                    s = os.path.split(s[0])
                    m = size_folder_re.match(s[1])

		#remove the trailing slash
		context = context[0:-1]
		if context not in self.context_list:
		    self.context_list.append(context)
		    self.emit('context-added', context)

		s = os.path.split(s[0]) #theme
		theme = s[1]

		icon = [theme, context, icon_name]
		if theme != self.icon_theme_name:
		    bg_color =  gtk.gdk.color_parse("#FFCCCC")
                else:
		    bg_color =  gtk.gdk.color_parse("#FFFFFF")
            
	    try:
	        pixbuf = icon_info.load_icon()
            except:
                continue

	    if pixbuf and size > big_size and size <= 48:
		big_size = size
		if pixbuf.get_width() > size or pixbuf.get_height() > size:
		    display_pixbuf = pixbuf.subpixbuf(0, 0, size, size)
		else:
		    display_pixbuf = pixbuf

                is_link = False
		if os.path.islink(icon_info.get_filename()):
                    is_link = True
                    link_height = self.link_emblem.get_height()
		    link_width = self.link_emblem.get_width()

		    x_att = display_pixbuf.get_width() -  link_width
                    y_att = display_pixbuf.get_height() - link_height


		    new_width = link_width + x_att
		    new_height = link_height + y_att
		    
		    if new_width < display_pixbuf.get_width():
		        new_width = display_pixbuf.get_width()
                    
		    if new_height < display_pixbuf.get_height():
		        new_height = display_pixbuf.get_height()

                    tmp_pixbuf = gtk.gdk.Pixbuf(display_pixbuf.get_colorspace(),
		                                display_pixbuf.get_has_alpha(),
						display_pixbuf.get_bits_per_sample(),
						new_width,
						new_height)

		    tmp_pixbuf.fill(0x00000000)

                    display_pixbuf.composite(tmp_pixbuf,
		                         0, 0,
					 display_pixbuf.get_width(),
					 display_pixbuf.get_height(),
					 0, 0,
					 1, 1,
					 gtk.gdk.INTERP_NEAREST,
					 255)

		    self.link_emblem.composite(tmp_pixbuf,
		                         x_att, 
					 y_att,
					 link_width,
					 link_height,
					 x_att, 
					 y_att,
					 1, 1,
					 gtk.gdk.INTERP_NEAREST,
					 255)

		    display_pixbuf = tmp_pixbuf

	    pixbuf_list[self.col_size_map[size]-self.COL_SIZE_START] = pixbuf

	if icon:
	    display_name_str+= ')'
	    #hack for right now

            is_new_link = False
            if self.icon_new_links.has_key((icon_name, context)):
	        is_new_link = True
		is_link = False
                (source_name, source_context, ext) = self.icon_new_links[(icon_name, context)]
                bg_color = gtk.gdk.color_parse("#CCCCFF")
                display_pixbuf = self.icon_theme.load_icon(source_name, big_size, 0)

	    icon.append(display_name_str)
	    icon.append(display_pixbuf)
	    icon.append(bg_color)
	    icon.append(is_link)
	    icon.append(is_new_link)
	    icon.append(ext)
	    icon.extend(pixbuf_list)

	    self.icon_model.append(icon)
      
        return True

    def output_link_file(self, context_map):
        file = open("link.xml", "w")
        file.write("""<?xml version="1.0" standalone="yes" ?>
<!DOCTYPE mapping SYSTEM "legacy-icon-mapping.dtd">

<mapping>
""")
        for context in context_map.keys():
	    mapping = context_map[context]
	    file.write('  <context dir="%s">\n'%context)

	    for source_name in mapping.keys():
                file.write('    <icon name="%s">\n'%source_name)
                for target in mapping[source_name]:
		    file.write('      <link>%s</link>\n'%target)

		file.write('    </icon>\n')
	    file.write('  </context>\n')
	        
        file.write('</mapping>')
	file.close()

    def process_links(self):
        file = './link.xml'
        if os.path.isfile(file):
            parser = make_parser()
            links_mapping_handler = LinkLoaderHandler()
            parser.setContentHandler(links_mapping_handler)
            parser.parse(file)
	    self.icon_new_links = links_mapping_handler.mapping
            print self.icon_new_links

    def generate_context_map(self):
        #put links list into a source->target mapping
	context_map = {}
        for target_key in self.icon_new_links.keys():
	    (target_name, target_context) = target_key
            (source_name, source_context, source_ext) = self.icon_new_links[target_key]

            if target_context == source_context:
	        target_link = target_name
            else:
	        target_link = '../%s/%s'%(target_context, target_name)

            if source_ext:
                target_link += "." + source_ext
 
	    if not context_map.has_key(source_context):
	        source_map = {source_name: [target_link]}
	        context_map[source_context] = source_map
            else:
	        source_map = context_map[source_context]
		if not source_map.has_key(source_name):
		    source_map[source_name] = [target_link]
		else:
		    target_list = source_map[source_name]
		    target_list.append(target_link)
        
	return context_map

    def link(self, target_path, source_path):

        target_iter = self.icon_model.get_iter(target_path)
        source_iter = self.icon_model.get_iter(source_path)

        target_icon, target_context = self.icon_model.get(target_iter, 
	                                                  self.COL_NAME,
							  self.COL_CONTEXT)

        source_icon, source_context, source_pixbuf, source_ext = self.icon_model.get(source_iter, 
	                                                                             self.COL_NAME,
							                             self.COL_CONTEXT,
									             self.COL_DISPLAY_PIXBUF,
										     self.COL_EXTENTION)
        key = (target_icon, target_context)

        self.icon_new_links[key] = (source_icon, source_context, source_ext)

	color = gtk.gdk.color_parse('#CCCCFF')

        self.icon_model.set(target_iter, 
	                    self.COL_BG_COLOR, color, 
			    self.COL_DISPLAY_PIXBUF, source_pixbuf,
			    self.COL_IS_LINK, False,
			    self.COL_IS_NEW_LINK, True)

    def run(self):
        screen = gtk.gdk.screen_get_default()
	self.icon_theme_name = screen.get_setting("gtk-icon-theme-name")
        self.icon_theme = gtk.icon_theme_get_for_screen(screen)
	self.unprocessed_icon_queue = list(self.icon_theme.list_icons())
	self.link_emblem = self.icon_theme.load_icon("emblem-symbolic-link", 24, 0)
        self.process_links()

	gobject.idle_add(self.process_icon_queue)

gobject.type_register(CurrentThemeProcessor)

class IconCoverageApp:
    def __init__(self):
        self.main_window = gtk.Window(gtk.WINDOW_TOPLEVEL) 
        self.main_window.set_default_size(640, 480)
	self.main_window.connect("destroy", self.destroy)
	self.main_window.set_title("Gtk Icon Theme Coverage Tool")
	self.scrolled_window_left = gtk.ScrolledWindow(None, None)
	self.scrolled_window_right = gtk.ScrolledWindow(None, None)
        self.icon_view_left = gtk.IconView()
	self.icon_view_right = gtk.IconView()
	self.scrolled_window_left.add(self.icon_view_left)
        self.scrolled_window_right.add(self.icon_view_right)
	self.paned = gtk.HPaned()
	self.controls_left = gtk.VBox()
        self.controls_right = gtk.VBox()
        self.info_left = gtk.VBox()
	self.info_right = gtk.VBox()

	vbox = gtk.VBox()
	vbox.pack_start(self.controls_left, False, False)
	vbox.pack_start(self.scrolled_window_left, True, True, 5)
        vbox.pack_start(self.info_left, False, False)
	self.paned.pack1(vbox, resize=True)

	vbox = gtk.VBox()
	vbox.pack_start(self.controls_right, False, False)
	vbox.pack_start(self.scrolled_window_right, True, True, 5)
        vbox.pack_start(self.info_right, False, False)
        self.paned.pack2(vbox)

        self.main_vbox = gtk.VBox(spacing=5)
	self.main_vbox.pack_end(self.paned)

	self.main_window.add(self.main_vbox)

    def populate_theme_filter(self, combobox, active_entry):
        combobox.append_text("All")
        combobox.append_text(self.theme_processor.icon_theme_name)
	combobox.append_text("Other")
        combobox.set_active(active_entry)


    def destroy(self, widget, data=None):
        gtk.main_quit()

    def run(self):
        self.theme_processor = CurrentThemeProcessor()
        self.theme_processor.run()

	icon_renderer = gtk.CellRendererPixbuf()
	icon_renderer.set_property("follow-state", True)
	self.icon_view_left.pack_start(icon_renderer)
	self.icon_view_left.set_attributes(icon_renderer,
	                                   pixbuf=self.theme_processor.COL_DISPLAY_PIXBUF,
					   cell_background_gdk=self.theme_processor.COL_BG_COLOR)

        self.icon_view_right.pack_start(icon_renderer)
	self.icon_view_right.set_attributes(icon_renderer,
	                                   pixbuf=self.theme_processor.COL_DISPLAY_PIXBUF,
					   cell_background_gdk=self.theme_processor.COL_BG_COLOR)

        self.filter_controls_left = FilterPane(self.theme_processor, self.icon_view_left)
        self.filter_controls_right = FilterPane(self.theme_processor, self.icon_view_right)

	self.controls_left.pack_start(self.filter_controls_left)
	self.controls_right.pack_start(self.filter_controls_right)	

        self.filter_controls_left.set_active_theme(0)
        self.filter_controls_right.set_active_theme(1)

        self.details_left = DetailsPane(self.theme_processor)
        self.details_right = DetailsPane(self.theme_processor)

        self.info_left.pack_start(self.details_left)
	self.info_right.pack_start(self.details_right)

        self.icon_view_left.connect('selection-changed', self.details_left.selection_changed)
        self.icon_view_right.connect('selection-changed', self.details_right.selection_changed)

        self.action_pane = ActionPane(self.theme_processor,
	                              self.icon_view_left,
				      self.icon_view_right)
	self.main_vbox.pack_start(self.action_pane, False, False)

        self.main_window.show_all()
	gtk.main()
